@@ -2,6 +2,32 @@ context(paste("Symbolic differentiation rules v", packageVersion("Deriv"), sep="
 lc_orig=Sys.getlocale(category = "LC_COLLATE")
 Sys.setlocale(category = "LC_COLLATE", locale = "C")
 
+num_test_deriv <- function(fun, larg, narg=1, h=1.e-5, tolerance=2000*h^2) {
+   # test the first derivative of a function fun() (given as a character
+   # string) by Deriv() and central difference.
+   # larg is a named list of parameters to pass to fun
+   # narg indicates by which of fun's arguments the differentiation must be made
+   # h is the small perturbation in the central differentiation: x-h and x+h 
+   # Parameter tolerance is used in comparison test.
+   if (length(names(larg)) == 0)
+      stop(sprintf("No argument for function %s() to differentiate. There must be at leat one argument.", fun))
+   if (h <= 0)
+      stop("Parameter h must be positive")
+   larg_ph=larg_mh=larg
+   larg_ph[[narg]]=larg_ph[[narg]]+h
+   larg_mh[[narg]]=larg_mh[[narg]]-h
+   f_ph=do.call(fun, larg_ph)
+   f_mh=do.call(fun, larg_mh)
+   dnum=(f_ph-f_mh)/(2*h)
+   sym_larg=larg
+   nm_x=names(larg)[narg]
+   sym_larg[[narg]]=as.symbol(nm_x)
+   flang=as.symbol(fun)
+   dsym=do.call(as.function(c(sym_larg, Deriv(as.call(c(flang, sym_larg)), nm_x))), larg, quote=TRUE)
+#cat(sprintf("comparing %s by %s\n", format1(as.call(c(flang, larg))), nm_x))
+   expect_equal(dnum, dsym, tolerance=tolerance, info=sprintf("%s by %s", format1(as.call(c(flang, larg))), nm_x))
+}
+
 f=function(x) {} # empty place holder
 
 expect_equal_deriv <- function(t, r, nmvar="x") {
@@ -65,7 +91,7 @@ test_that("elementary functions", {
    expect_equal_deriv(exp(x), exp(x))
    expect_equal_deriv(expm1(x), exp(x))
    expect_equal_deriv(log(x), 1/x)
-   expect_equal_deriv(log1p(x), 1/(x+1))
+   expect_equal_deriv(log1p(x), 1/(1+x))
    expect_equal_deriv(abs(x), sign(x))
    expect_equal_deriv(sign(x), 0)
    expect_equal_deriv(sinh(x), cosh(x))
@@ -88,29 +114,29 @@ test_that("special functions", {
    expect_equal_deriv(besselI(x, 1), 0.5 * (besselI(x, 0) + besselI(x, 2)))
    expect_equal_deriv(besselI(x, 1, FALSE), 0.5 * (besselI(x, 0) + besselI(x, 2)))
    expect_equal_deriv(besselI(x, 1, TRUE), 0.5 * (besselI(x, 0, TRUE) + besselI(x, 2, TRUE))-besselI(x, 1, TRUE))
-   expect_equal_deriv(besselI(x, n), if (n == 0) besselI(x, 1) else 0.5 * (besselI(x, n + 1) + besselI(x, n - 1)))
-   expect_equal_deriv(besselI(x, n, TRUE), (if (n == 0) besselI(x, 1, TRUE) else 0.5 * (besselI(x, n + 1, TRUE) + besselI(x, n - 1, TRUE)))-besselI(x, n, TRUE))
+   expect_equal_deriv(besselI(x, n), if (n == 0) besselI(x, 1) else 0.5 * (besselI(x, 1 + n) + besselI(x, n - 1)))
+   expect_equal_deriv(besselI(x, n, TRUE), (if (n == 0) besselI(x, 1, TRUE) else 0.5 * (besselI(x, 1 + n, TRUE) + besselI(x, n - 1, TRUE)))-besselI(x, n, TRUE))
    expect_equal_deriv(besselK(x, 0), -besselK(x, 1))
    expect_equal_deriv(besselK(x, 0, FALSE), -besselK(x, 1))
    expect_equal_deriv(besselK(x, 0, TRUE), besselK(x, 0, TRUE)-besselK(x, 1, TRUE))
    expect_equal_deriv(besselK(x, 1), -(0.5 * (besselK(x, 0) + besselK(x, 2))))
    expect_equal_deriv(besselK(x, 1, FALSE), -(0.5 * (besselK(x, 0) + besselK(x, 2))))
    expect_equal_deriv(besselK(x, 1, TRUE), besselK(x, 1, TRUE)-0.5 * (besselK(x, 0, TRUE) + besselK(x, 2, TRUE)))
-   expect_equal_deriv(besselK(x, n), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, n + 1) + besselK(x, n - 1))))
-   expect_equal_deriv(besselK(x, n, FALSE), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, n + 1) + besselK(x, n - 1))))
-   expect_equal_deriv(besselK(x, n, TRUE), besselK(x, n, TRUE)+if (n == 0) -besselK(x, 1, TRUE) else -(0.5 * (besselK(x, n + 1, TRUE) + besselK(x, n - 1, TRUE))))
+   expect_equal_deriv(besselK(x, n), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, 1 + n) + besselK(x, n - 1))))
+   expect_equal_deriv(besselK(x, n, FALSE), if (n == 0) -besselK(x, 1) else -(0.5 * (besselK(x, 1 + n) + besselK(x, n - 1))))
+   expect_equal_deriv(besselK(x, n, TRUE), besselK(x, n, TRUE)+if (n == 0) -besselK(x, 1, TRUE) else -(0.5 * (besselK(x, 1 + n, TRUE) + besselK(x, n - 1, TRUE))))
    expect_equal_deriv(besselJ(x, 0), -besselJ(x, 1))
    expect_equal_deriv(besselJ(x, 1), 0.5 * (besselJ(x, 0) - besselJ(x, 2)))
-   expect_equal_deriv(besselJ(x, n), if (n == 0) -besselJ(x, 1) else 0.5 * (besselJ(x, n - 1) - besselJ(x, n + 1)))
+   expect_equal_deriv(besselJ(x, n), if (n == 0) -besselJ(x, 1) else 0.5 * (besselJ(x, n - 1) - besselJ(x, 1 + n)))
    expect_equal_deriv(besselY(x, 0), -besselY(x, 1))
    expect_equal_deriv(besselY(x, 1), 0.5 * (besselY(x, 0) - besselY(x, 2)))
-   expect_equal_deriv(besselY(x, n), if (n == 0) -besselY(x, 1) else 0.5 * (besselY(x, n - 1) - besselY(x, n + 1)))
+   expect_equal_deriv(besselY(x, n), if (n == 0) -besselY(x, 1) else 0.5 * (besselY(x, n - 1) - besselY(x, 1 + n)))
    expect_equal_deriv(gamma(x), digamma(x) * gamma(x))
    expect_equal_deriv(lgamma(x), digamma(x))
    expect_equal_deriv(digamma(x), trigamma(x))
    expect_equal_deriv(trigamma(x), psigamma(x, 2L))
    expect_equal_deriv(psigamma(x), psigamma(x, 1L))
-   expect_equal_deriv(psigamma(x, n), psigamma(x, n+1L))
+   expect_equal_deriv(psigamma(x, n), psigamma(x, 1L+n))
    expect_equal_deriv(beta(x, y), beta(x, y) * (digamma(x) - digamma(x + y)))
    expect_equal_deriv(beta(x, y), beta(x, y) * (digamma(y) - digamma(x + y)), "y")
    expect_equal_deriv(lbeta(x, y), digamma(x) - digamma(x + y))
@@ -152,6 +178,46 @@ test_that("expression cache test", {
    expect_equal(g2n(x, m, s), g2c(x, m, s))
 })
 
+# test error reporting
+test_that("error reporting", {
+   expect_error(Deriv(rnorm), "is not in derivative table", fixed=TRUE)
+   expect_error(Deriv(~rnorm(x), "x"), "is not in derivative table", fixed=TRUE)
+   expect_error(Deriv(~x+rnorm(x), "x"), "is not in derivative table", fixed=TRUE)
+})
+
+# systematic central difference tests
+set.seed(7)
+test_that("central differences", {
+   for (nm_f in ls(drule)) {
+      rule <- drule[[nm_f]]
+      larg <- rule
+      narg <- length(larg)
+      larg[] <- runif(narg)
+      # possible logical parameters are swithed on/off
+      fargs=formals(nm_f)
+      ilo=sapply(fargs, is.logical)
+      if (any(ilo))
+         logrid=do.call(expand.grid, rep(list(c(TRUE, FALSE)), sum(ilo)))
+      for (iarg in seq_len(narg)) {
+         if (is.null(rule[[iarg]]))
+            next
+         if (is.null(fargs) || !any(ilo)) {
+            suppressWarnings(num_test_deriv(nm_f, larg, narg=iarg))
+         } else {
+            apply(logrid, 1, function(lv) {
+               lolarg=larg
+               lolarg[ilo]=lv
+               suppressWarnings(num_test_deriv(nm_f, lolarg, narg=iarg))
+            })
+         }
+      }
+   }
+})
+
+tmp <- Deriv(Deriv(quote(dnorm(x ** 2 - x)), "x"), "x")
+test_that("dsym cleaning after nested call", {
+   expect_identical(Deriv(quote(.e1*x), "x"), quote(.e1)) # was issue #2
+})
 
 # doc examples
 fsq <- function(x) x^2
@@ -160,7 +226,7 @@ f_ <- Deriv(fsc)
 fc <- function(x, h=0.1) if (abs(x) < h) 0.5*h*(x/h)**2 else abs(x)-0.5*h
 myfun <- function(x, y=TRUE) NULL # do something usefull
 dmyfun <- function(x, y=TRUE) NULL # myfun derivative by x.
-drule[["myfun"]] <- qlist(x=dmyfun(x, y), y=NULL) # y is just a logical
+drule[["myfun"]] <- alist(x=dmyfun(x, y), y=NULL) # y is just a logical
 #cat("Deriv(myfun)=", format1(Deriv(myfun)), "\n")
 
 test_that("doc examples", {
@@ -174,4 +240,5 @@ test_that("doc examples", {
    expect_equal(Deriv(fc, "x", cache=FALSE), function(x, h=0.1) if (abs(x) < h) x/h else sign(x))
    expect_equal(Deriv(myfun(z^2, FALSE), "z"), quote(2 * (z * dmyfun(z^2, FALSE))))
 })
+drule[["myfun"]] <- NULL
 Sys.setlocale(category = "LC_COLLATE", locale = lc_orig)
