@@ -16,6 +16,8 @@ test_that("rational simplifications", {
    expect_equal_lang(a/a, 1) # complete single simplification
    expect_equal_lang(2/2, 1) # complete single numeric simplification
    expect_equal_lang(a*b/(b*a), 1) # complete multiple simplification
+   expect_equal_lang(a/(b*a^x), a^(1 - x)/b) # diff numeric - symbol
+   expect_equal_lang(a^x/(b*a), a^(x - 1)/b) # diff symbol - numeric
    expect_equal_lang(a/(b*a), 1/b) # single simplification in denominator
    expect_equal_lang(-a/(b*a), -(1/b)) # single negative simplification in denominator
    expect_equal_lang(2/(b*2), 1/b) # single numeric simplification in denominator
@@ -35,6 +37,7 @@ test_that("log simplifications", {
    expect_equal_lang(log(exp(a)), a)
    expect_equal_lang(log(a^n), n*log(a))
    expect_equal_lang(log(sqrt(a)), 0.5*log(a))
+   expect_equal_lang(log(1+x), log1p(x))
 })
 test_that("sqrt simplifications", {
    expect_equal_lang(sqrt(a), sqrt(a)) # no change must occur
@@ -58,7 +61,26 @@ test_that("factorizations", {
    expect_equal_lang(1+x-1, x)
 })
 test_that("term order", {
-   expect_equal_lang(a(x+1)+a(x-1), a(x+1)+a(x-1)) # no change must occur
-   expect_equal_lang(a(x-1)+a(x+1), a(x+1)+a(x-1)) # "+" is before "-" in C collate. It is inverse in fr_FR.UTF8
+   expect_equal_lang(a(1+x)+a(x-1), a(1+x)+a(x-1)) # no change must occur
+   expect_equal_lang(a(x-1)+a(x+1), a(1+x)+a(x-1)) # "+" is before "-" in C collate. It is inverse in fr_FR.UTF8
+   expect_equal_lang(1+a, 1+a) # no change must occur
+   expect_equal_lang(a+1, 1+a)
+})
+test_that("Cache test", {
+   expect_equal(Cache(quote(c(a+b, sin(a+b)))), quote({
+    .e1 <- a + b
+    c(.e1, sin(.e1))
+    }
+    ))
+   expect_equal(Cache(quote({t=a+b; c(a+b, sin(a+b))})), quote({
+    t = a + b
+    c(t, sin(t))
+    }
+    ))
+   expect_equal(Simplify(quote({t=a+b; c(a+b, sin(a+b))})), quote({
+    t = a + b
+    c(t, sin(t))
+    }
+    ))
 })
 Sys.setlocale(category = "LC_COLLATE", locale = lc_orig)
